#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Feb  9 14:21:56 2022
@author: David ALBERTO (www.astrolabe-science.fr)
"""
import matplotlib.pyplot as plt
import datetime as dt
import numpy as np
import ephem

#coordonnées de la comète de Halley (code du logiciel XEphem,avec les paramètres orbitaux)
# Source : http://astro.vanbuitenen.nl/cometelements?format=xephem&mag=obs

line="1P/Halley,e,162.2239,58.9763,111.9047,17.872265,0.0130447,0.966321,171.4710,02/10.0/2022,2000,g 4.0,6.0"
halley = ephem.readdb(line)#attention : pas de tirets dans le nom de la comète

angle=0.934#trouvé par tâtonnements, pour rendre horizontal le demi grand-axe de l'ellipse
#cet angle est à adapter à l'époque choisie pour les dates.
# L'ellipse tracée sert de guide pour ajuster cet angle

plt.rcParams["font.family"] = "Ubuntu"

# -------------------Création de la liste de dates :

#Liste de dates au 1er janvier de l'année :
listeDates=[dt.datetime(1985+N,1,1) for N in range(76)]

fig=plt.figure(figsize=(8,3),tight_layout=True)
ax=plt.subplot()
ax.set_aspect('equal')
ax.set_xticks([])
ax.set_yticks([])
ax.set_ylim(-7,7)
ax.set_title("Position de la comète de Halley (%i - %i)"%(listeDates[0].year,listeDates[-1].year))

ax.scatter(0,0,s=10,c='orange')

#Tracé de l'ellipse (expression mathématique):----------------------
a=17.872265#1/2 grand axe
e=0.966321#excentricité
c=e*a#abscisse du foyer
b=np.sqrt(a**2-c**2)#demi petit-axe
x=np.arange(-2*a,2*a,0.01)
y=b*np.sqrt(1-(x+c)**2/a**2)
plt.plot(x,y,c='gray',lw=0.2,zorder=-2)
plt.plot(x,-y,c='gray',lw=0.2,zorder=-2)
# ---------------------------------------------------------
#Tracé des positions de la comète :
taillePoint=6
couleur_comete='royalblue'
prop1=dict(s=taillePoint,c=couleur_comete,edgecolor='k',lw=0.2,zorder=0)#format du point année
prop2=dict(s=2*taillePoint,c='orangered',edgecolor='k',lw=0.5,zorder=1)#format du point 5 ans

for date in listeDates:
    halley.compute(date)
    R=halley.sun_distance
    theta=halley.hlon#longitude écliptique
    # Passage aux coordonnées cartésiennes :
    X=R*np.cos(theta+angle)
    Y=R*np.sin(theta+angle)
    plt.scatter(X,Y,**prop1)
    if int(date.year)%5==0:#pour les années en multiple de 5 : étiquette de l'année
    # calcul du décalage des étiquettes par rapport aux points
        X_offset=(X+c)/abs(X+c)#décalage +1 si X>-c, sinon -1
        Y_offset=Y/abs(Y)#décalage +1 si Y positif, sinon -1
        plt.annotate(date.year,xy=(X,Y),xytext=(X+X_offset,Y+Y_offset),fontsize=6,arrowprops=dict(arrowstyle="-", linewidth=0.5,color='gray'))
        plt.scatter(X,Y,**prop2)

#orbite terrestre :
circle=plt.Circle((0,0),1,facecolor='none',edgecolor='gray',lw=0.2,ls='--',zorder=-1)
ax.add_patch(circle)
plt.scatter(np.cos(2.5),np.sin(2.5),s=3,c='royalblue')#Terre
plt.text(np.cos(2.5),np.sin(2.5),"T",ha='right', fontsize=5,c='royalblue')

#Échelle des distances
props = dict(boxstyle='round', facecolor='wheat', pad=0.5,linewidth=0.5)
plt.text(-.5*a, 0, "18 u.a.", fontsize=6, bbox=props, ha='center',va='center')

plt.plot([-c,a-c],[0,0],lw=0.5,markevery=[0,-1],marker='|',color='gray',zorder=-1)#axe horizontal
ax.text(0.0,-0.01,"D. Alberto (www.astrolabe-science.fr)",fontsize=5,va='top',transform=ax.transAxes)
ax.text(1,0.01,r"Période $\simeq$ 75 à 76 ans ",c='crimson',fontsize=8,ha='right',transform=ax.transAxes)
# -------------------------------------------------------
fig.savefig("TrajectoireCometeHalley.pdf")
fig.savefig("TrajectoireCometeHalley.png",dpi=300)

plt.show()